# -*- coding: utf-8 -*-

import os

"""Main module."""


def unpack(filename):

    return {
        k : v
        for k, v in [
            str.split('=', 1) # maxsplit=1 
            if '=' in str else (str, '')
            for str in os.path.basename(filename).split('+')
        ] + [ ('_', filename) ]
    }


def pack(dict):

    regular = {}
    underscore = {}
    ext = {}

    stringified = {str(k): str(v) for k, v in dict.items()}
    stringified.pop('_', None)

    for k, v in stringified.items():

        assert not any(c in k or c in v for c in ('=', '+'))

        if k == 'ext':
            ext[k] = v
        elif k.startswith('_'):
            underscore[k] = v
        else:
            regular[k] = v

    return '+'.join([ k + '=' + v
        for k, v in
        sorted(regular.items())
        + sorted(underscore.items())
        + sorted(ext.items())
    ])



# -*- coding: utf-8 -*-

"""Console script for keyname."""
import sys
import click
import keyname as kn

@click.group()
def main():
    pass

@main.command()
def version(args=None):
    """Print version information and exit."""
    click.echo( kn.__version__ )
    return 0

@main.command()
@click.argument('keys', nargs=-1)
def keep(keys):
    """Filter stdin keyname string to keep argument keys."""
    click.echo( kn.pack( {
        k : v for k, v in kn.unpack( input() ).items() if k in keys
    } ) )
    return 0

@main.command()
@click.argument('keys', nargs=-1)
def drop(keys):
    """Filter stdin keyname string to drop argument keys."""
    click.echo( kn.pack( {
        k : v for k, v in kn.unpack( input() ).items() if k not in keys
    } ) )
    return 0

@main.command(
    context_settings={
        'ignore_unknown_options' : True,
        'allow_extra_args' : True,
    },
)
@click.pass_context
def pack(ctx):
    """Combine option-argument pairs into a keyname string."""
    # adapted from https://stackoverflow.com/a/32946412
    click.echo( kn.pack({
        ctx.args[i][2:] : ctx.args[i+1]
        for i in range(0, len(ctx.args), 2)
    }) )
    return 0

@main.command()
def keys():
    """Print keys from stdin keyname string."""
    for k, v in kn.unpack( input() ).items():
        if '_' != k:
            click.echo( k )
    return 0

@main.command()
def vals():
    """Print values from stdin keyname string."""
    for k, v in kn.unpack( input() ).items():
        if '_' != k:
            click.echo( v )
    return 0

@main.command()
@click.argument('keys', nargs=-1)
def extract(keys):
    """Print values corresponding to keys in stdin keyname string."""
    keyname_attributes = kn.unpack( input() )
    for k in keys:
        if k in keyname_attributes:
            click.echo( keyname_attributes[ k ] )
    return 0

if __name__ == "__main__":
    sys.exit(cli())  # pragma: no cover



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `keyname` package."""


import unittest
from click.testing import CliRunner

from keyname import keyname as kn
from keyname import cli


class TestKeyname(unittest.TestCase):
    """Tests for `keyname` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_unpack(self):
        """Test unpacking."""
        goal = {
            'seed' : '100',
            'foobar' : '20',
            '_hash' : 'asdf',
            'ext' : '.txt'
        }

        name = "seed=100+foobar=20+_hash=asdf+ext=.txt"
        goal['_'] = name
        assert kn.unpack(name) == goal

        # reorderings
        name = "foobar=20+seed=100+_hash=asdf+ext=.txt"
        goal['_'] = name
        assert kn.unpack(name) == goal

        name = "_hash=asdf+foobar=20+seed=100+ext=.txt"
        goal['_'] = name
        assert kn.unpack(name) == goal

        # should ignore path
        name = "path/seed=100+foobar=20+_hash=asdf+ext=.txt"
        goal['_'] = name
        assert kn.unpack(name) == goal

        name = "~/more=path/+blah/seed=100+foobar=20+_hash=asdf+ext=.txt"
        goal['_'] = name
        assert kn.unpack(name) == goal

        name = "just/a/regular/file.pdf"
        assert kn.unpack(name) == {
            'file.pdf' : '',
            '_' : 'just/a/regular/file.pdf'
        }

        name = "key/with/no+=value/file+ext=.pdf"
        assert kn.unpack(name) == {
            'file' : '',
            'ext' : '.pdf',
            '_' : 'key/with/no+=value/file+ext=.pdf'
        }

        name = "multiple/=s/file=biz=blah+ext=.pdf"
        assert kn.unpack(name) == {
            'file' : 'biz=blah',
            'ext' : '.pdf',
            '_' : 'multiple/=s/file=biz=blah+ext=.pdf'
        }


    def test_001_pack(self):
        """Test packing."""

        # reorderings
        assert kn.pack({
             'seed' : '100',
             'foobar' : '20',
             '_hash' : 'asdf',
             'ext' : '.txt'
         }) == "foobar=20+seed=100+_hash=asdf+ext=.txt"

        assert kn.pack({
             '_hash' : 'asdf',
             'seed' : '100',
             'foobar' : '20',
             'ext' : '.txt'
         }) == "foobar=20+seed=100+_hash=asdf+ext=.txt"

        assert kn.pack({
             '_hash' : 'asdf',
             'foobar' : '20',
             'ext' : '.txt',
             'seed' : '100'
         }) == "foobar=20+seed=100+_hash=asdf+ext=.txt"

        # different values
        assert kn.pack({
             'seed' : '100',
             'foobar' : 'blip',
             '_hash' : 'asdf',
             'ext' : '.txt'
         }) == "foobar=blip+seed=100+_hash=asdf+ext=.txt"

        assert kn.pack({
             'seed' : 'a100',
             'foobar' : 'blip',
             '_hash' : 'asdf',
             'ext' : '.txt'
         }) == "foobar=blip+seed=a100+_hash=asdf+ext=.txt"

        assert kn.pack({
             'aseed' : 'a100',
             'foobar' : 'blip',
             '_hash' : 'asdf',
             'ext' : '.txt'
         }) == "aseed=a100+foobar=blip+_hash=asdf+ext=.txt"

        # should ignore '_' key
        assert kn.pack({
             'seed' : '100',
             'foobar' : '20',
             '_hash' : 'asdf',
             'ext' : '.txt',
             '_' : 'foobar=20+seed=100+_hash=asdf+ext=.txt'
         }) == "foobar=20+seed=100+_hash=asdf+ext=.txt"

        assert kn.pack({
             'seed' : '100',
             'foobar' : '20',
             '_hash' : 'asdf',
             'ext' : '.txt',
             '_' : 'path/seed=100+foobar=20+_hash=asdf+ext=.txt'
         }) == "foobar=20+seed=100+_hash=asdf+ext=.txt"

        assert kn.pack({
             'seed' : '100',
             'foobar' : '20',
             '_hash' : 'asdf',
             'ext' : '.txt',
             '_' : '~/more=path/+blah/seed=100+foobar=20+_hash=asdf+ext=.txt'
         }) == "foobar=20+seed=100+_hash=asdf+ext=.txt"

        assert kn.pack({
             'seed' : '100',
             'foobar' : '20',
             '_hash' : 'asdf',
             'ext' : '.txt',
             '_' : '"whatever+=/"'
         }) == "foobar=20+seed=100+_hash=asdf+ext=.txt"

        # missing extension
        assert kn.pack({
             '_hash' : 'asdf',
             'foobar' : '20',
             'seed' : '100',
         }) == "foobar=20+seed=100+_hash=asdf"



    def test_command_line_interface(self):
        """Test the CLI."""
        runner = CliRunner()
        result = runner.invoke(cli.main)
        assert result.exit_code == 0
        assert 'Usage' in result.output
        help_result = runner.invoke(cli.main, ['--help'])
        assert help_result.exit_code == 0
        assert '--help  Show this message and exit.' in help_result.output
